package com.tzld.longarticle.recommend.server.service;

import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.model.*;
import com.tzld.longarticle.recommend.server.repository.crawler.PublishContentSortLogRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.PublishSortLogRepository;
import com.tzld.longarticle.recommend.server.repository.entity.crawler.PublishContentSortLog;
import com.tzld.longarticle.recommend.server.repository.entity.crawler.PublishSortLog;
import com.tzld.longarticle.recommend.server.repository.mapper.crawler.ArticleUserGroupMapper;
import com.tzld.longarticle.recommend.server.service.rank.RankParam;
import com.tzld.longarticle.recommend.server.service.rank.RankResult;
import com.tzld.longarticle.recommend.server.service.rank.RankService;
import com.tzld.longarticle.recommend.server.service.recall.RecallParam;
import com.tzld.longarticle.recommend.server.service.recall.RecallResult;
import com.tzld.longarticle.recommend.server.service.recall.RecallService;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import com.tzld.longarticle.recommend.server.util.DateUtils;
import com.tzld.longarticle.recommend.server.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import static com.tzld.longarticle.recommend.server.service.SceneConstants.FWH_COLD_START;

/**
 * @author dyp
 */
@Service
@Slf4j
public class RecommendService {

    @Autowired
    private RecallService recallService;
    @Autowired
    private RankService rankService;
    @Autowired
    private PublishContentSortLogRepository publishContentSortLogRepository;
    @Autowired
    private PublishSortLogRepository publishSortLogRepository;
    @Autowired
    AccountIndexAvgViewCountService accountIndexAvgViewCountService;
    @Autowired
    private ArticleUserGroupMapper articleUserGroupMapper;

    @ApolloJsonValue("${accountStrategyConfig:{}}")
    private Map<String, String> accountStrategyConfigMap;
    @Value("${spring.profiles.active}")
    private String env;


    public RecommendResponse recommend(RecommendRequest request) {

        RecommendParam param = genRecommendParam(request, SceneConstants.DEFAULT);
        log.info("genRecommendParam {}", JSONUtils.toJson(param));
        // 获取账号排序设置
        String strategyConfig = accountStrategyConfigMap.get(request.getAccountName());
        if (StringUtils.hasText(strategyConfig) && !request.isParamStrategy()) {
            param.setStrategy(strategyConfig);
        }
        RecallResult recallResult = recallService.recall(convertToRecallParam(param));

        RankResult rankResult = rankService.rank(convertToRankParam(param, recallResult));
        //log.info("rankResult {}", JSONUtils.toJson(rankResult));

        saveSortLog(param, rankResult);

        return buildRecommendResponse(recallResult, rankResult, param.getPublishNum());
    }

    public RecommendWithUserGroupResponse recommend4FwhColdStart(RecommendRequest request) {

        RecommendParam param = genRecommendParam(request, FWH_COLD_START);
        log.info("genRecommendParam {}", JSONUtils.toJson(param));
        // 获取账号排序设置
        String strategyConfig = accountStrategyConfigMap.get(request.getAccountName());
        if (StringUtils.hasText(strategyConfig)) {
            param.setStrategy(strategyConfig);
        }
        RecallResult recallResult = recallService.recall(convertToRecallParam(param));
        //log.info("recallResult {}", JSONUtils.toJson(recallResult));

        RankResult rankResult = rankService.rank(convertToRankParam(param, recallResult));
        //log.info("rankResult {}", JSONUtils.toJson(rankResult));

        saveSortLog(param, rankResult);


        List<RecommendWithUserGroupResponse.RankData> rankList = new ArrayList<>();
        List<Content> contentList = rankResult.getContents();
        for (int i = 0, start = 0; i < param.getUserGroupIds().size() && start < contentList.size(); i++) {
            RecommendWithUserGroupResponse.RankData rankData = new RecommendWithUserGroupResponse.RankData();
            rankData.setUserGroupId(param.getUserGroupIds().get(i));
            int end = Math.min(contentList.size(), start + param.getPublishNum());
            rankData.setContentIds(CommonCollectionUtils.toList(contentList.subList(start, end), Content::getId));
            rankList.add(rankData);
            start += param.getPublishNum();
        }

        List<RecommendWithUserGroupResponse.FilterData> filterList = new ArrayList<>();
        for (RecallResult.RecallData data : recallResult.getData()) {
            if (CollectionUtils.isNotEmpty(data.getFilterContents())) {
                for (Content filterContent : data.getFilterContents()) {
                    RecommendWithUserGroupResponse.FilterData filterData = new RecommendWithUserGroupResponse.FilterData();
                    filterData.setContentId(filterContent.getId());
                    filterData.setFilterReason(filterContent.getFilterReason());
                    filterList.add(filterData);
                }
            }
        }


        return new RecommendWithUserGroupResponse(0, "success",
                new RecommendWithUserGroupResponse.RecommendWithUserGroupData(rankList, filterList));
    }

    private RecommendResponse buildRecommendResponse(RecallResult recallResult, RankResult rankResult, Integer publishNum) {
        List<Content> contentList = rankResult.getContents();
        List<ArticleSortResponseDataItem> rankList = new ArrayList<>();
        List<ArticleSortResponseDataItem> filterContentList = new ArrayList<>();
        for (RecallResult.RecallData data : recallResult.getData()) {
            if (CollectionUtils.isNotEmpty(data.getFilterContents())) {
                for (Content filterContent : data.getFilterContents()) {
                    ArticleSortResponseDataItem item = new ArticleSortResponseDataItem();
                    BeanUtils.copyProperties(filterContent, item);
                    filterContentList.add(item);
                }
            }
        }
        ArticleSortResponseData data = new ArticleSortResponseData();
        if (CollectionUtils.isNotEmpty(contentList)) {
            for (Content content : contentList) {
                ArticleSortResponseDataItem item = new ArticleSortResponseDataItem();
                BeanUtils.copyProperties(content, item);
                rankList.add(item);
            }
        }
        data.setRank_list(rankList.subList(0, Math.min(rankList.size(), publishNum)));
        data.setFilter_list(filterContentList);

        RecommendResponse response = new RecommendResponse();
        response.setCode(0);
        response.setMsg("success");
        response.setData(data);
        return response;
    }

    public RecommendParam genRecommendParam(RecommendRequest request, String abCode) {
        RecommendParam param = new RecommendParam();
        BeanUtils.copyProperties(request, param);
        param.setScene(abCode);

        switch (abCode) {
            case FWH_COLD_START:
                List<Integer> userGroupIds = articleUserGroupMapper.selectUserGroupIdByGzhId(request.getGhId());
                LocalDate today = LocalDate.now(); // 获取当前日期
                int dayOfYear = today.getDayOfYear();
                int tailNum = dayOfYear % 8;
                List<Integer> needPushUserGroupIds = new ArrayList<>();
                for (int userGroupId : userGroupIds) {
                    if (userGroupId % 8 == tailNum) {
                        needPushUserGroupIds.add(userGroupId);
                    }
                }
                param.setUserGroupIds(needPushUserGroupIds);
                break;
        }

        return param;
    }

    public RecallParam convertToRecallParam(RecommendParam param) {
        RecallParam recallParam = new RecallParam();
        BeanUtils.copyProperties(param, recallParam);
        recallParam.setStrategy(param.getStrategy());
        recallParam.setGhId(param.getGhId());
        return recallParam;
    }


    public RankParam convertToRankParam(RecommendParam param, RecallResult recallResult) {
        RankParam rankParam = new RankParam();
        List<Content> contentList = new ArrayList<>();
        List<Content> backup = new ArrayList<>();
        if (Objects.nonNull(recallResult) && CollectionUtils.isNotEmpty(recallResult.getData())) {
            for (RecallResult.RecallData item : recallResult.getData()) {
                if (item.isBackup()) {
                    backup.addAll(item.getContents());
                } else {
                    contentList.addAll(item.getContents());
                }
            }

        }
        rankParam.setContents(contentList);
        rankParam.setBackup(backup);
        rankParam.setStrategy(param.getStrategy());
        rankParam.setGhId(param.getGhId());
        rankParam.setAccountName(param.getAccountName());
        rankParam.setSize(param.getPublishNum());
        rankParam.setScene(param.getScene());
        rankParam.setUserGroupIds(param.getUserGroupIds());

        return rankParam;
    }

    /**
     * 记录 账号、排序策略、结果。。。
     *
     * @param param
     * @param rankResult
     */
    private void saveSortLog(RecommendParam param, RankResult rankResult) {
        if (!"prod".equals(env) || param.isExcludeLog()) {
            return;
        }
        switch (param.getScene()) {
            case FWH_COLD_START:
                break;
            default:
                PublishContentSortLog log = new PublishContentSortLog();
                log.setGhId(param.getGhId());
                log.setAccountName(param.getAccountName());
                log.setStrategy(param.getStrategy());
                List<String> publishContentIds = rankResult.getContents().stream().map(Content::getId).collect(Collectors.toList());
                log.setPublishContentId(JSONObject.toJSONString(publishContentIds));
                log.setCreateTimestamp(System.currentTimeMillis());
                publishContentSortLogRepository.save(log);
                break;

        }
        List<PublishSortLog> publishSortLogSaveList = new ArrayList<>();
        for (int i = 1; i < rankResult.getContents().size() + 1; i++) {
            Content content = rankResult.getContents().get(i - 1);

            PublishSortLog sortLog = new PublishSortLog();
            sortLog.setDateStr(DateUtils.getCurrentDateStr("yyyy-MM-dd"));
            sortLog.setGhId(param.getGhId());
            sortLog.setAccountName(param.getAccountName());
            sortLog.setCrawlerChannelContentId(content.getCrawlerChannelContentId());
            sortLog.setTitle(content.getTitle());
            sortLog.setIndex(i);
            sortLog.setIndexAvgCount(accountIndexAvgViewCountService.getAvgReadCountByDB(param.getGhId(), i));
            sortLog.setCategory(JSONObject.toJSONString(content.getCategory()));
            sortLog.setStrategy(param.getStrategy());
            sortLog.setCreateTimestamp(System.currentTimeMillis());
            publishSortLogSaveList.add(sortLog);
        }
        publishSortLogRepository.saveAll(publishSortLogSaveList);
    }

}
